@@ -1,3 +1,4 @@
+use super::attributes::FieldAttributes;
 use std::str::FromStr;
 
 use proc_macro2::{Ident, Span, TokenStream};
@@ -5,16 +6,13 @@ use quote::ToTokens;
 use quote::TokenStreamExt;
 use syn::parse::{Parse, ParseStream, Result};
 use syn::spanned::Spanned;
-use syn::{
-    AttrStyle, Data, DeriveInput, Expr, Fields, GenericParam, Generics, Token, Type, Visibility,
-};
+use syn::{AttrStyle, Data, DeriveInput, Fields, GenericParam, Generics, Token, Type, Visibility};
 
-#[derive(Clone)]
 pub struct Field {
     pub vis: Visibility,
     pub ident: Ident,
     pub ty: Type,
-    pub expr: Option<Expr>,
+    pub attrs: FieldAttributes,
 }
 
 pub struct StructureInput {
@@ -44,21 +42,18 @@ impl Parse for StructureInput {
         let mut optional_fields: Vec<Field> = vec![];
         let mut required_fields: Vec<Field> = vec![];
         for f in fields.named.into_iter() {
-            // Having "default" attribute
-            match f.attrs.iter().find(|attr| attr.path.is_ident("default")) {
-                Some(attr) => optional_fields.push(Field {
-                    vis: f.vis,
-                    ident: f.ident.unwrap(),
-                    ty: f.ty,
-                    expr: Some(attr.parse_args().unwrap()),
-                }),
-                None => required_fields.push(Field {
-                    vis: f.vis,
-                    ident: f.ident.unwrap(),
-                    ty: f.ty,
-                    expr: None,
-                }),
+            let attrs: FieldAttributes = f.attrs.into();
+            let fields = if attrs.default.is_some() {
+                &mut optional_fields
+            } else {
+                &mut required_fields
             };
+            fields.push(Field {
+                vis: f.vis,
+                ident: f.ident.unwrap(),
+                ty: f.ty,
+                attrs,
+            });
         }
         Ok(StructureInput {
             vis,
@@ -75,9 +70,11 @@ impl ToTokens for StructureInput {
         let ident = &self.ident;
         let vis = &self.vis;
 
+        // Parse generic parameters.
         let impl_tokens = self.tokenize_impl();
         let ty_tokens = self.tokenize_types();
-        let (_, ty_generics, where_clause) = self.generics.split_for_impl();
+
+        let where_clause = &self.generics.where_clause;
         let lifetimes = self
             .generics
             .lifetimes()
@@ -89,7 +86,6 @@ impl ToTokens for StructureInput {
 
         let all_generics = self.all_generics().collect::<Vec<TokenStream>>();
         let empty_generics = self.empty_generics();
-
         let optional_generics = self.optional_generics();
         let satisfied_generics = self.satified_generics();
 
@@ -105,7 +101,7 @@ impl ToTokens for StructureInput {
                 _phantom: ::std::marker::PhantomData<(#ty_tokens #(#all_generics),*)>,
                 #(#builder_fields),*
             }
-            impl <#impl_tokens> #ident #ty_generics #where_clause {
+            impl <#impl_tokens> #ident <#(#lifetimes,)* #ty_tokens> #where_clause {
                 #vis fn new() -> #builder_name<#(#lifetimes,)* #ty_tokens #(#empty_generics),*> {
                     #builder_name {
                         _phantom: ::std::marker::PhantomData,
@@ -116,7 +112,7 @@ impl ToTokens for StructureInput {
             impl <#impl_tokens #(#optional_generics,)*> #builder_name <#(#lifetimes,)* #ty_tokens #(#satisfied_generics),*>
                 #where_clause
             {
-                #vis fn build(self) -> #ident #ty_generics {
+                #vis fn build(self) -> #ident <#(#lifetimes,)* #ty_tokens> {
                     #ident {
                         #(#struct_init_args),*
                     }
@@ -132,7 +128,7 @@ impl StructureInput {
     fn all_generics(&self) -> impl Iterator<Item = TokenStream> {
         (0..(self.required_fields.len() + self.optional_fields.len()))
             .into_iter()
-            .map(|i| TokenStream::from_str(&format!("U{}", i + 1)).unwrap())
+            .map(|i| TokenStream::from_str(&format!("TyBuilderPattern{}", i + 1)).unwrap())
     }
 
     /// An iterator to describe initial state of builder.
@@ -145,65 +141,65 @@ impl StructureInput {
     /// An iterator for optional fields.
     fn optional_generics(&self) -> impl Iterator<Item = TokenStream> {
         let offset = self.required_fields.len() + 1;
-        (0..self.optional_fields.len())
-            .into_iter()
-            .map(move |i| TokenStream::from_str(&format!("U{}", i + offset)).unwrap())
+        (0..self.optional_fields.len()).into_iter().map(move |i| {
+            TokenStream::from_str(&format!("TyBuilderPattern{}", i + offset)).unwrap()
+        })
     }
 
     /// An iterator to describe when the builder has enough types to build the struct.
-    fn satified_generics<'a>(&'a self) -> impl 'a + Iterator<Item = TokenStream> {
+    fn satified_generics(&'_ self) -> impl '_ + Iterator<Item = TokenStream> {
         self.required_fields
             .iter()
             .map(|f| {
                 let ty = &f.ty;
-                TokenStream::from(quote! {#ty})
+                quote! {#ty}
             })
             .chain(self.optional_generics())
     }
 
     /// An iterator for fields of the builder.
-    fn builder_fields<'a>(&'a self) -> impl 'a + Iterator<Item = TokenStream> {
+    fn builder_fields(&'_ self) -> impl '_ + Iterator<Item = TokenStream> {
         let iters = self
             .required_fields
             .iter()
             .chain(self.optional_fields.iter());
         iters.map(|f| {
             let (ident, ty) = (&f.ident, &f.ty);
-            TokenStream::from(quote! {
+            quote! {
                 #ident: Option<#ty>
-            })
+            }
         })
     }
 
     /// An iterator for initialize arguments of the builder.
     /// Required fields are filled with `None`, optional fields are filled with given value via `default` attribute.
-    fn builder_init_args<'a>(&'a self) -> impl 'a + Iterator<Item = TokenStream> {
+    fn builder_init_args(&'_ self) -> impl '_ + Iterator<Item = TokenStream> {
         self.required_fields
             .iter()
             .map(|f| {
                 let ident = &f.ident;
-                TokenStream::from(quote! {
+                quote! {
                     #ident: None
-                })
+                }
             })
             .chain(self.optional_fields.iter().map(|f| {
-                let (ident, expr) = (&f.ident, &f.expr);
-                TokenStream::from(quote_spanned! { expr.span() =>
+                let (ident, expr) = (&f.ident, &f.attrs.default.as_ref());
+                quote_spanned! { expr.span() =>
                     #ident: Some(#expr)
-                })
+                }
             }))
     }
 
     /// An iterator to express initialize statements.
-    fn struct_init_args<'a>(&'a self) -> impl 'a + Iterator<Item = TokenStream> {
+    fn struct_init_args(&'_ self) -> impl '_ + Iterator<Item = TokenStream> {
         self.required_fields
             .iter()
             .chain(self.optional_fields.iter())
             .map(|f| {
                 let ident = &f.ident;
-                TokenStream::from(quote! {
+                quote! {
                     #ident: self.#ident.unwrap()
-                })
+                }
             })
     }
 
@@ -211,7 +207,7 @@ impl StructureInput {
     fn builder_functions<'a>(
         &'a self,
         builder_name: &'a Ident,
-        lifetimes: &'a Vec<TokenStream>,
+        lifetimes: &'a [TokenStream],
         ty_tokens: &'a TokenStream,
     ) -> impl 'a + Iterator<Item = TokenStream> {
         let vis = &self.vis;
@@ -224,7 +220,7 @@ impl StructureInput {
             .chain(self.optional_fields.iter())
             .map(|f| {
                 let ident = &f.ident;
-                TokenStream::from(quote! { #ident: self.#ident })
+                quote! { #ident: self.#ident }
             })
             .collect::<Vec<TokenStream>>();
 
@@ -239,25 +235,50 @@ impl StructureInput {
                 let mut before_generics = all_generics.clone();
                 before_generics[index] = TokenStream::from_str("()").unwrap();
                 let mut after_generics = all_generics.clone();
-                after_generics[index] = TokenStream::from(quote! {#ty});
+                after_generics[index] = quote! {#ty};
                 let mut builder_fields = all_builder_fields.clone();
-                builder_fields[index] = TokenStream::from(quote! {#ident: Some(value.into())});
-                index = index + 1;
-                TokenStream::from(quote! {
+                builder_fields[index] = quote! {#ident: Some(value.into())};
+                index += 1;
+                
+                let (arg_type_gen, arg_type) = 
+                    if f.attrs.use_into {
+                        (Some(quote!{<IntoType: Into<#ty>>}), TokenStream::from_str("IntoType").unwrap())
+                    } else {
+                        (None, quote! {#ty})
+                    };
+                let ret_expr = quote! {
+                    #builder_name {
+                        _phantom: ::std::marker::PhantomData,
+                        #(#builder_fields),*
+                    }
+                };
+                let (ret_type, ret_expr) = match &f.attrs.validator {
+                    Some(v) => (quote! {
+                        ::std::result::Result< #builder_name <#(#lifetimes,)* #ty_tokens #(#after_generics),*>, ()>
+                    }, quote_spanned! { v.span() =>
+                        match #v (value) {
+                            ::std::result::Result::Ok(value) => ::std::result::Result::Ok(#ret_expr),
+                            ::std::result::Result::Err(_) => ::std::result::Result::Err(())
+                        }
+                    }),
+                    None => (quote! {
+                        #builder_name <#(#lifetimes,)* #ty_tokens #(#after_generics),*>
+                    }, ret_expr)
+                };
+                quote! {
                     impl <#impl_tokens #(#other_generics,)*> #builder_name <#(#lifetimes,)* #ty_tokens #(#before_generics),*>
                         #where_clause
                     {
-                        #vis fn #ident<InToType: Into<#ty>>(mut self, value: InToType) -> #builder_name <#(#lifetimes,)* #ty_tokens #(#after_generics),*> {
-                            #builder_name {
-                                _phantom: ::std::marker::PhantomData,
-                                #(#builder_fields),*
-                            }
+                        #vis fn #ident #arg_type_gen(mut self, value: #arg_type) -> #ret_type {
+                            #ret_expr
                         }
                     }
-                })
+                }
             })
     }
 
+    /// Tokenize type parameters.
+    /// It skips lifetimes and has no outer brackets.
     fn tokenize_types(&self) -> TokenStream {
         let generics = &self.generics;
         let mut tokens = TokenStream::new();
